
library(CSTools)
library(ClimProjDiags)
library(s2dv)
library(qmap)
library(CSIndicators)

climatologies_era5 <- function(obs_uas, obs_vas, obs_uas_100, obs_vas_100, mask_ori, dates_obs, config, ncores = 1){

  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/GetProbs_2.R')

  output <- list()
  obs_uas <- drop(obs_uas)
  obs_vas <- drop(obs_vas)
  obs_uas <- s2dv::Reorder(data = obs_uas, order = c('year','time'))
  obs_vas <- s2dv::Reorder(data = obs_vas, order = c('year','time'))
  # Remove 29 Feb
  indices_remove <- apply(dates_obs, 1, function(x) {which(is.na(x) == T)})
  if (config$years[1] %% 4 == 0 && (config$years[1] %% 100 != 0 || config$years[1] %% 400 == 0)){
    keep_indices <- !(seq_len(dim(obs_uas)['time']) %in% indices_remove[[2]])
  } else {
    keep_indices <- !(seq_len(dim(obs_uas)['time']) %in% indices_remove[[1]])
  }

  sfc_wind_obs <- sqrt(obs_uas^2 + obs_vas^2)
  sfc_wind_obs <- ClimProjDiags::Subset(x = sfc_wind_obs, along = 'time', indices = keep_indices, drop = FALSE)
  wind_100 <- ClimProjDiags::Subset(x = wind_100, along = 'time', indices = keep_indices, drop = FALSE)
  dates_obs <- ClimProjDiags::Subset(x = dates_obs, along = 'time', indices = keep_indices, drop = FALSE)
  multiplication <- ifelse(mask_ori == 0, 1.39, 1.29)
  multiplication <- array(multiplication, dim = dim(sfc_wind_obs))
  sfc_wind_obs <- sfc_wind_obs * multiplication
  wind_100 <- sqrt(obs_uas_100^2 + obs_vas_100^2)

  months <- format(dates_dcpp, "%m")
  seasons <- list(
    DJF = c("12", "01", "02"),
    MAM = c("03", "04", "05"),
    JJA = c("06", "07", "08"),
    SON = c("09", "10", "11"),
    Annual = c('01','02','03','04','05','06','07','08','09','10','11','12')
  )
  seasons_masks <- lapply(seasons, function(seasons_months) {
    format(dates_first_year_au, "%m") %in% seasons_months
  })
  
# 
#   output$wind_100_DJF <- array(NA, dim = c(sdate_dim))
#   output$wind_100_JJA <- array(NA, dim = c(sdate_dim))
#   output$wind_100_SON <- array(NA, dim = c(sdate_dim))
#   output$wind_100_Annual <- array(NA, dim = c(sdate_dim))
#   output$sfc_wind_DJF <- array(NA, dim = c(sdate_dim))
#   output$sfc_wind_MAM <- array(NA, dim = c(sdate_dim))
#   output$sfc_wind_JJA <- array(NA, dim = c(sdate_dim))
#   output$sfc_wind_SON <- array(NA, dim = c(sdate_dim))
#   output$sfc_wind_Annual <- array(NA, dim = c(sdate_dim))

  for (season in names(seasons)){
    seasons_mask <- seasons_masks[[season]]
    sfc_wind_season <- sfc_wind[,seasons_mask]
    sfc_wind_season <- MeanDims(data = sfc_wind_season, dims = c('year','time'))
    wind_100_season <- wind_100[,seasons_mask]
    wind_100_season <- MeanDims(data = wind_100_season, dims = c('year','time'))
    if (season == 'DJF'){
      output$sfc_wind_DJF <- sfc_wind_season
      output$wind_100_DJF <- wind_100_season
    } else if (season == 'MAM'){
      output$sfc_wind_MAM <- sfc_wind_season
      output$wind_100_MAM <- wind_100_season
    } else if (season == 'JJA'){
      output$sfc_wind_JJA <- sfc_wind_season
      output$wind_100_JJA <- wind_100_season
    } else if (season == 'SON'){
      output$sfc_wind_SON <- sfc_wind_season
      output$wind_100_SON <- wind_100_season
    } else if (season == 'Annual'){
      output$sfc_wind_Annual <- sfc_wind_season
      output$wind_100_Annual <- wind_100_season
    }
  }

  return(output)
}
