
library(startR)
library(s2dv)
library(CSTools)

compute_metrics <- function(dcpp, obs, model, dcpp_probs, obs_probs, time_dim = 'year'){
  output <- list()
  output$acc <- CorrEno(exp = dcpp, obs = obs, time_dim = time_dim, member_dim = 'member', method = 'pearson', test.type = 'one-sided', alpha = config$alpha, pval = FALSE, ncores = config$ncores)
  output$rmsss <- RMSSS_edited(exp = dcpp, obs = obs, ref = NULL, ref_value = 0, test.type = 'greater', alpha = 0.05, time_dim = time_dim, member_dim = 'member', ncores = config$ncores)
  print('rmsss,acc done ')
  if (model %in% c('Multi-model-mean','Multi-model-mean-oper')){
    dcpp_probs <- multiApply::Apply(data = dcpp_probs, target_dims = 'member', fun = mean, na.rm = FALSE, ncores = config$ncores)$output1
    clim_probs <- array(data = 1/config$n_categories, dim = dim(dcpp_probs))
    rps_dcpp <- multiApply::Apply(data = list(probs_exp = dcpp_probs, probs_obs = obs_probs), target_dims = 'bin', fun = .RPS_from_probs, n_categories = config$n_categories, Fair = FALSE, n_members = NULL, ncores = config$ncores)$output1
    rps_clim <- multiApply::Apply(data = list(probs_exp = clim_probs, probs_obs = obs_probs), target_dims = 'bin', fun = .RPS_from_probs, n_categories = config$n_categories, Fair = FALSE, n_members = NULL, ncores = config$ncores)$output1
    output$rpss <- list()
    output$rpss$rpss <- 1 - multiApply::Apply(data = rps_dcpp, target_dims = time_dim, fun = mean, na.rm = FALSE, ncores = config$ncores)$output1 / multiApply::Apply(data = rps_clim, target_dims = time_dim, fun = mean, na.rm = FALSE, ncores = config$ncores)$output1
    output$rpss$sign <- RandomWalkTest(skill_A = rps_dcpp, skill_B = rps_clim, time_dim = time_dim, test.type = 'greater', alpha = config$alpha, ncores = config$ncores)$signif
    rm(dcpp_aux_probs,dcpp_probs,obs_probs,rps_dcpp,rps_clim)
  } else {
    # output$rpss <- RPSS(exp = dcpp, obs = obs, ref = NULL, n_categories = config$n_categories, indices_for_clim = indices_for_clim, test.type = 'greater', alpha = config$alpha, Fair = FALSE, time_dim = time_dim, member_dim = 'member', ncores = config$ncores)
    print('rpss done')
  }
  # output$crpss <- CRPSS(exp = dcpp, obs = obs, ref = NULL, test.type = 'greater', alpha = config$alpha, Fair = FALSE, time_dim = time_dim, memb_dim = 'member', dat_dim = NULL, ncores = config$ncores)
  # output$rocss <- ROCSS(exp = dcpp, obs = obs, ref = NULL, n_categories = config$n_categories, time_dim = time_dim, member_dim = 'member', ncores = config$ncores)
  # output$spread_error <- easyVerification::veriApply(verifun = 'EnsSprErr', fcst = dcpp, obs = obs, tdim = which(names(dim(dcpp))==time_dim), ensdim = which(names(dim(dcpp))=='member'), na.rm = FALSE, ncpus = 1)
  
  return(output)
}



