
library(CSTools)
library(ClimProjDiags)
library(s2dv)

daily_means_3hr <- function(data, dates, config, ncores = 1){
  indices_remove <- apply(dates, 1, function(x) {which(is.na(x) == T)})
  if (config$years[1] %% 4 == 0 && (config$years[1] %% 100 != 0 || config$years[1] %% 400 == 0)){
    keep_indices <- !(seq_len(dim(data)['time']) %in% indices_remove[[2]])
  } else {
    keep_indices <- !(seq_len(dim(data)['time']) %in% indices_remove[[1]])
  }
  data <- ClimProjDiags::Subset(x = data, along = 'time', indices = keep_indices, drop = FALSE)
  dates <- ClimProjDiags::Subset(x = dates, along = 'time', indices = keep_indices, drop = FALSE)
  months <- format(dates, "%m")
  seasons <- list(
    DJF = c("12", "01", "02"),
    MAM = c("03", "04", "05"),
    JJA = c("06", "07", "08"),
    SON = c("09", "10", "11"),
    Annual = c('01','02','03','04','05','06','07','08','09','10','11','12')
  )
  dates_first_year_au <- dates[1,]
  seasons_masks <- lapply(seasons, function(seasons_months) {
    format(dates_first_year_au, "%m") %in% seasons_months
  })
  output <- list()
  dates_djf <- dates[1,seasons_masks[['DJF']]]
  days <- format(dates_djf, "%Y-%m-%d")
  unique_days <- unique(days)
  n_days <- length(unique_days)
  output$direct_djf <- array(data = NA, dim = c(year = length(config$years), time = n_days))
  output$hr3_djf <- array(data = NA, dim = c(year = length(config$years), time = n_days))
  dates_mam <- dates[1,seasons_masks[['MAM']]]
  days <- format(dates_mam, "%Y-%m-%d")
  unique_days <- unique(days)
  n_days <- length(unique_days)
  output$direct_mam <- array(data = NA, dim = c(year = length(config$years), time = n_days))
  output$hr3_mam <- array(data = NA, dim = c(year = length(config$years), time = n_days))
  dates_jja <- dates[1,seasons_masks[['JJA']]]
  days <- format(dates_jja, "%Y-%m-%d")
  unique_days <- unique(days)
  n_days <- length(unique_days)
  output$direct_jja <- array(data = NA, dim = c(year = length(config$years), time = n_days))
  output$hr3_jja <- array(data = NA, dim = c(year = length(config$years), time = n_days))
  dates_son <- dates[1,seasons_masks[['SON']]]
  days <- format(dates_son, "%Y-%m-%d")
  unique_days <- unique(days)
  n_days <- length(unique_days)
  output$direct_son <- array(data = NA, dim = c(year = length(config$years), time = n_days))
  output$hr3_son <- array(data = NA, dim = c(year = length(config$years), time = n_days))
  days <- format(dates[1,], "%Y-%m-%d")
  unique_days <- unique(days)
  n_days <- length(unique_days)
  output$direct <- array(data = NA, dim = c(year = length(config$years), time = n_days))
  output$hr3 <- array(data = NA, dim = c(year = length(config$years), time = n_days))
  for (yy in 1:length(config$years)){
    if (yy == 1){
      
    }
    data_year <- data[yy,]
    dates_year <- dates[yy,]
    for (season in names(seasons)){
      print(season)
      seasons_mask <- seasons_masks[[season]]
      d_seas <- data_year[seasons_mask]
      dates_seas <- dates_year[seasons_mask]
      dates_seas <- as.Date(dates_seas)
      # daily accumulated
      daily_acc <- tapply(d_seas, dates_seas, sum, na.rm = TRUE)
      daily_acc <- unname(daily_acc)

      # 3 hourly means + accumulated
      grp3    <- ceiling(seq_along(d_seas) / 3)
      means3h <- tapply(d_seas, grp3, mean, na.rm = TRUE)
      blocks_per_day <- 24 / 3
      day_idx <- ceiling(seq_along(means3h) / blocks_per_day)
      daily_acc_from_3h <- tapply(means3h, day_idx, function(x) sum(x * 3, na.rm = TRUE))
      daily_acc_from_3h <- unname(daily_acc_from_3h)
browser()
      if (season == 'DJF'){
        output$direct_djf[yy,] <- daily_acc
        output$hr3_djf[yy,] <- daily_acc_from_3h
      } else if (season == 'MAM'){
        output$direct_mam[yy,] <- daily_acc
        output$hr3_mam[yy,] <- daily_acc_from_3h
      } else if (season == 'JJA'){
        output$direct_jja[yy,] <- daily_acc
        output$hr3_jja[yy,] <- daily_acc_from_3h
      } else if (season == 'SON'){
        output$direct_son[yy,] <- daily_acc
        output$hr3_son[yy,] <- daily_acc_from_3h
      } else if (season == 'Annual'){
        output$direct[yy,] <- daily_acc
        output$hr3[yy,] <- daily_acc_from_3h
      }
    }
  }
  for (name in names(output)) {
    dims <- dim(output[[name]])
    if (!is.null(dims) && length(dims) == 2) {
      dimnames(output[[name]]) <- list(
        year = as.character(config$years),
        time = as.character(seq_len(dims[2]))
      )
    }
  }
  return(output)
}

