
library(ncdf4)
library(s2dv)

load_mask <- function(grid, mask_path = NULL, land_value = 0, sea_value = 1, crop = FALSE){
  
  if (is.null(mask_path)){
    mask_sea_land_path = '/esarchive/exp/ecearth/constant/land_sea_mask_512x256.nc'
  } else if (is.character(mask_path)){
    mask_sea_land_path = mask_path
  } else {
    stop("mask_path must be NULL (to use the default mask and interpolate it to the specified grid) or a string with the mask's path you want to load")
  }
  
  mask_sea_land = ncdf4::nc_open(mask_sea_land_path)
  lon_mask_sea_land = ncdf4::ncvar_get(mask_sea_land,'lon')
  lat_mask_sea_land = ncdf4::ncvar_get(mask_sea_land,'lat')
  mask_sea_land = ncdf4::ncvar_get(mask_sea_land,'LSM')
  names(dim(mask_sea_land)) = c('lon','lat')
  
  mask_interp = s2dv::CDORemap(data_array = mask_sea_land,
                               lons = lon_mask_sea_land,
                               lats = lat_mask_sea_land,
                               grid = grid, 
                               method = 'bil', 
                               crop = crop)
  
  mask = NULL
  mask$mask = mask_interp$data_array
  mask$mask[mask_interp$data_array <=0] = sea_value
  mask$mask[mask_interp$data_array > 0] = land_value
  mask$mask = s2dv::Reorder(data = mask$mask, order = c('lon','lat'))
  mask$lat = mask_interp$lats
  mask$lon = mask_interp$lons
  
  return(mask)
}
