library(CSTools)
library(ClimProjDiags)
library(s2dv)
library(qmap)

solar_capacity_factors <- function(obs_tas, obs_rsds, tas, rsds, agregation, dates_obs, dates_dcpp, fyears, model, method, config, ncores = 1){

  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/make_eval_train_dexes_mine.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/GetProbs_2.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/add_window_dim.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/Quantile_Mapping_fake.R')
  source('/esarchive/scratch/smoreno/gitlab/cstools/R/CST_QuantileMapping.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/CFSLR.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/PV_pot.R')
  
  output <- list()
  obs_tas <- drop(obs_tas)
  obs_rsds <- drop(obs_rsds)
  obs_tas <- s2dv::Reorder(data = obs_tas, order = c('year','time'))
  obs_rsds <- s2dv::Reorder(data = obs_rsds, order = c('year','time'))
  names(dim(obs_tas))[1] <- 'sdate'
  names(dim(obs_rsds))[1] <- 'sdate'
  indices_remove <- apply(dates_obs, 1, function(x) {which(is.na(x) == T)})
  # Remove 29 Feb from observations
  if (config$years[1] %% 4 == 0 && (config$years[1] %% 100 != 0 || config$years[1] %% 400 == 0)){
    keep_indices <- !(seq_len(dim(obs_tas)['time']) %in% indices_remove[[2]])
  } else {
    keep_indices <- !(seq_len(dim(obs_tas)['time']) %in% indices_remove[[1]])
  }
  obs_tas <- ClimProjDiags::Subset(x = obs_tas, along = 'time', indices = keep_indices, drop = FALSE)
  obs_tas <- multiApply::Apply(data = obs_tas, target_dims = 'sdate', 
                               fun = function(data){zoo::rollmean(x = as.vector(data), k = length(fyears), fill = NULL, align = 'left')}, 
                               output_dims = 'sdate', ncores = config$ncores)$output1
  obs_rsds <- ClimProjDiags::Subset(x = obs_rsds, along = 'time', indices = keep_indices, drop = FALSE)
  obs_rsds <- multiApply::Apply(data = obs_rsds, target_dims = 'sdate', 
                               fun = function(data){zoo::rollmean(x = as.vector(data), k = length(fyears), fill = NULL, align = 'left')}, 
                               output_dims = 'sdate', ncores = config$ncores)$output1
  dates_obs <- ClimProjDiags::Subset(x = dates_obs, along = 'time', indices = keep_indices, drop = FALSE)
  # Remove extra days from obs for this model
  if (model == 'HadGEM3-GC3.1-MM'){
    obs_tas <- obs_tas[,,,1:360]
    obs_rsds <- obs_rsds[,,,1:360]
  } 
  dcpp_tas <- drop(tas)
  dcpp_rsds <- drop(rsds)
  sdate_dim <- dim(obs_rsds)['sdate']
  cross <- make_eval_train_dexes_mine(eval.method = 'leave-one-out', amt.points = sdate_dim,
                                  amt.points_cor = NULL, k = 1, tail.out = FALSE)
  print('Calibration')
  months <- format(dates_obs, "%m")
  seasons <- list(
    DJF = c("12", "01", "02"),
    MAM = c("03", "04", "05"),
    JJA = c("06", "07", "08"),
    SON = c("09", "10", "11"),
    Annual = c('01','02','03','04','05','06','07','08','09','10','11','12')
  )
  dates_first_year_au <- dates_obs[format(dates_obs, "%Y") == config$sdates[2]]
  seasons_masks <- lapply(seasons, function(seasons_months) {
    format(dates_first_year_au, "%m") %in% seasons_months
  })

  output$pv_dcpp_DJF <- array(NA, dim = c(sdate_dim))
  output$pv_dcpp_MAM <- array(NA, dim = c(sdate_dim))
  output$pv_dcpp_JJA <- array(NA, dim = c(sdate_dim))
  output$pv_dcpp_SON <- array(NA, dim = c(sdate_dim))
  output$pv_dcpp_Annual <- array(NA, dim = c(sdate_dim))
  output$pv_obs_DJF <- array(NA, dim = c(sdate_dim))
  output$pv_obs_MAM <- array(NA, dim = c(sdate_dim))
  output$pv_obs_JJA <- array(NA, dim = c(sdate_dim))
  output$pv_obs_SON <- array(NA, dim = c(sdate_dim))
  output$pv_obs_Annual <- array(NA, dim = c(sdate_dim))
  output$pv_dcpp_probs_DJF <- array(NA, dim = c(bin = 3, sdate_dim))
  output$pv_dcpp_probs_MAM <- array(NA, dim = c(bin = 3, sdate_dim))
  output$pv_dcpp_probs_JJA <- array(NA, dim = c(bin = 3, sdate_dim))
  output$pv_dcpp_probs_SON <- array(NA, dim = c(bin = 3, sdate_dim))
  output$pv_dcpp_probs_Annual <- array(NA, dim = c(bin = 3, sdate_dim))
  output$n_effective_dcpp_DJF <- array(NA, dim = c(sdate_dim))
  output$n_effective_dcpp_MAM <- array(NA, dim = c(sdate_dim))
  output$n_effective_dcpp_JJA <- array(NA, dim = c(sdate_dim))
  output$n_effective_dcpp_SON <- array(NA, dim = c(sdate_dim))
  output$n_effective_dcpp_Annual <- array(NA, dim = c(sdate_dim))
  output$n_effective_obs_DJF <- array(NA, dim = c(sdate_dim))
  output$n_effective_obs_MAM <- array(NA, dim = c(sdate_dim))
  output$n_effective_obs_JJA <- array(NA, dim = c(sdate_dim))
  output$n_effective_obs_SON <- array(NA, dim = c(sdate_dim))
  output$n_effective_obs_Annual <- array(NA, dim = c(sdate_dim))
  
    for (t in 1:sdate_dim){
      print(t)
      obs_tas_tr <- ClimProjDiags::Subset(x = obs_tas, along = 'sdate', indices = cross[[t]]$train.dexes, drop = 'selected')
      obs_rsds_tr <- ClimProjDiags::Subset(x = obs_rsds, along = 'sdate', indices = cross[[t]]$train.dexes, drop = 'selected')
      obs_tas_ev <- ClimProjDiags::Subset(x = obs_tas, along = 'sdate', indices = cross[[t]]$eval.dexes, drop = FALSE)
      obs_rsds_ev <- ClimProjDiags::Subset(x = obs_rsds, along = 'sdate', indices = cross[[t]]$eval.dexes, drop = FALSE)
      dcpp_tas_tr <- ClimProjDiags::Subset(x = dcpp_tas, along = 'sdate', indices = cross[[t]]$train.dexes, drop = 'selected')
      dcpp_rsds_tr <- ClimProjDiags::Subset(x = dcpp_rsds, along = 'sdate', indices = cross[[t]]$train.dexes, drop = 'selected')
      dcpp_tas_ev <- ClimProjDiags::Subset(x = dcpp_tas, along = 'sdate', indices = cross[[t]]$eval.dexes, drop = FALSE)
      dcpp_rsds_ev <- ClimProjDiags::Subset(x = dcpp_rsds, along = 'sdate', indices = cross[[t]]$eval.dexes, drop = FALSE)
      obs_rsds_tr_wi <- add_window_dim(obs_rsds_tr)
      dcpp_rsds_tr_wi <- add_window_dim(dcpp_rsds_tr)

      dcpp_rsds_ev <- Quantile_Mapping_fake(exp = dcpp_rsds_tr_wi, obs = obs_rsds_tr_wi,
                                               exp_cor = dcpp_rsds_ev,
                                               na.rm = TRUE,  memb_dim = NULL, sdate_dim = 'sdate',
                                               window_dim = 'window',
                                               ncores = config$ncores, method = 'QUANT', qstep = 0.01)

      dcpp_rsds_tr <- Quantile_Mapping_fake(exp = dcpp_rsds_tr_wi, obs = obs_rsds_tr_wi,
                                               exp_cor = dcpp_rsds_tr,
                                               na.rm = TRUE,  memb_dim = NULL, sdate_dim = 'sdate',
                                               window_dim = 'window',
                                               ncores = config$ncores, method = 'QUANT', qstep = 0.01)

      dcpp_rsds_ev <- s2dv::Reorder(data = dcpp_rsds_ev, order = c('sdate','time'))
      rm(dcpp_rsds_tr_wi,obs_rsds_tr_wi)
      obs_tas_tr_wi <- add_window_dim(obs_tas_tr)
      dcpp_tas_tr_wi <- add_window_dim(dcpp_tas_tr)
      dcpp_tas_ev <- Quantile_Mapping_fake(exp = dcpp_tas_tr_wi, obs = obs_tas_tr_wi,
                                        exp_cor = dcpp_tas_ev,
                                        na.rm = TRUE,  memb_dim = NULL, sdate_dim = 'sdate',
                                        window_dim = 'window',
                                        ncores = config$ncores, method = 'QUANT', qstep = 0.01)
      dcpp_tas_tr <- Quantile_Mapping_fake(exp = dcpp_tas_tr_wi, obs = obs_tas_tr_wi,
                                     exp_cor = dcpp_tas_tr,
                                     na.rm = TRUE,  memb_dim = NULL, sdate_dim = 'sdate',
                                     window_dim = 'window',
                                     ncores = config$ncores, method = 'QUANT', qstep = 0.01)
      
      rm(dcpp_tas_tr_wi,obs_tas_tr_wi)
      dcpp_tas_ev <- s2dv::Reorder(data = dcpp_tas_ev, order = c('sdate','time'))
      if (agregation == 'seasons'){
        if (t == 1){
          pv_dcpp_DJF <- array(NA, dim = c(sdate_dim))
          pv_dcpp_MAM <- array(NA, dim = c(sdate_dim))
          pv_dcpp_JJA <- array(NA, dim = c(sdate_dim))
          pv_dcpp_SON <- array(NA, dim = c(sdate_dim))
          pv_dcpp_Annual <- array(NA, dim = c(sdate_dim))
          pv_dcpp_probs_DJF <- array(NA, dim = c(bin = 3, sdate_dim))
          pv_dcpp_probs_MAM <- array(NA, dim = c(bin = 3, sdate_dim))
          pv_dcpp_probs_JJA <- array(NA, dim = c(bin = 3, sdate_dim))
          pv_dcpp_probs_SON <- array(NA, dim = c(bin = 3, sdate_dim))
          pv_dcpp_probs_Annual <- array(NA, dim = c(bin = 3, sdate_dim))
          pv_obs_DJF <- array(NA, dim = c(sdate_dim))
          pv_obs_MAM <- array(NA, dim = c(sdate_dim))
          pv_obs_JJA <- array(NA, dim = c(sdate_dim))
          pv_obs_SON <- array(NA, dim = c(sdate_dim))
          pv_obs_Annual <- array(NA, dim = c(sdate_dim))
          n_effective_dcpp_DJF <- array(NA, dim = c(sdate_dim))
          n_effective_dcpp_MAM <- array(NA, dim = c(sdate_dim))
          n_effective_dcpp_JJA <- array(NA, dim = c(sdate_dim))
          n_effective_dcpp_SON <- array(NA, dim = c(sdate_dim))
          n_effective_dcpp_Annual <- array(NA, dim = c(sdate_dim))
          n_effective_obs_DJF <- array(NA, dim = c(sdate_dim))
          n_effective_obs_MAM <- array(NA, dim = c(sdate_dim))
          n_effective_obs_JJA <- array(NA, dim = c(sdate_dim))
          n_effective_obs_SON <- array(NA, dim = c(sdate_dim))
          n_effective_obs_Annual <- array(NA, dim = c(sdate_dim))
        }
        for (season in names(seasons)){
          print(season)
          seasons_mask <- seasons_masks[[season]]
          dcpp_tas_ev_sea <- dcpp_tas_ev[,seasons_mask]
          dcpp_tas_ev_sea <- s2dv::InsertDim(data = dcpp_tas_ev_sea, posdim = 1, lendim = 1, name = 'sdate')
          names(dim(dcpp_tas_ev_sea))[2] <- 'time'
          dcpp_tas_tr_sea <- dcpp_tas_tr[,seasons_mask]
          obs_tas_ev_sea <- obs_tas_ev[,seasons_mask]
          obs_tas_ev_sea <- s2dv::InsertDim(data = obs_tas_ev_sea, posdim = 1, lendim = 1, name = 'sdate')
          names(dim(obs_tas_ev_sea))[2] <- 'time'
          obs_tas_tr_sea <- obs_tas_tr[,seasons_mask]
          dcpp_rsds_ev_sea <- dcpp_rsds_ev[,seasons_mask]
          dcpp_rsds_ev_sea <- s2dv::InsertDim(data = dcpp_rsds_ev_sea, posdim = 1, lendim = 1, name = 'sdate')
          names(dim(dcpp_rsds_ev_sea))[2] <- 'time'
          dcpp_rsds_tr_sea <- dcpp_rsds_tr[,seasons_mask]
          obs_rsds_ev_sea <- obs_rsds_ev[,seasons_mask]
          obs_rsds_ev_sea <- s2dv::InsertDim(data = obs_rsds_ev_sea, posdim = 1, lendim = 1, name = 'sdate')
          names(dim(obs_rsds_ev_sea))[2] <- 'time'
          obs_rsds_tr_sea <- obs_rsds_tr[,seasons_mask]
          n_effective_dcpp <- apply((dcpp_rsds_ev_sea > 208) & (dcpp_tas_ev_sea < 318), 1, function(x) sum(x, na.rm = TRUE))
          n_effective_obs <- apply((obs_rsds_ev_sea > 208) & (obs_tas_ev_sea < 318), 1, function(x) sum(x, na.rm = TRUE))
          
          if (method == 'first'){
            pv_dcpp <- multiApply::Apply(data = list(dcpp_tas_ev_sea, dcpp_rsds_ev_sea), target_dims = c('sdate','time'), fun =  solar2CF, ncores = config$ncores)$output1
            pv_dcpp_tr <- multiApply::Apply(data = list(dcpp_tas_tr_sea, dcpp_rsds_tr_sea), target_dims = c('sdate','time'), fun =  solar2CF, ncores = config$ncores)$output1
            pv_obs <- multiApply::Apply(data = list(obs_tas_ev_sea, obs_rsds_ev_sea), target_dims = c('sdate','time'), fun =  solar2CF, ncores = config$ncores)$output1
            pv_obs_tr <- multiApply::Apply(data = list(obs_tas_tr_sea, obs_rsds_tr_sea), target_dims = c('sdate','time'), fun =  solar2CF, ncores = config$ncores)$output1
          } else if (method == 'second'){
            pv_dcpp <- multiApply::Apply(data = list(dcpp_tas_ev_sea, dcpp_rsds_ev_sea), target_dims = c('sdate','time'), fun =  PV_pot, ncores = config$ncores)$output1
            pv_dcpp_tr <- multiApply::Apply(data = list(dcpp_tas_tr_sea, dcpp_rsds_tr_sea), target_dims = c('sdate','time'), fun =  PV_pot, ncores = config$ncores)$output1
            pv_obs <- multiApply::Apply(data = list(obs_tas_ev_sea, obs_rsds_ev_sea), target_dims = c('sdate','time'), fun =  PV_pot, ncores = config$ncores)$output1
            pv_obs_tr <- multiApply::Apply(data = list(obs_tas_tr, obs_rsds_tr), target_dims = c('sdate','time'), fun =  PV_pot, ncores = config$ncores)$output1
          }
          pv_dcpp <- s2dv::MeanDims(data = pv_dcpp, dims = 'time', na.rm = TRUE)
          pv_dcpp_tr <- s2dv::MeanDims(data = pv_dcpp_tr, dims = 'time', na.rm = TRUE)
          pv_obs <- s2dv::MeanDims(data = pv_obs, dims = 'time', na.rm = TRUE)
          
          lims_ano_obs_tr <- multiApply::Apply(pv_obs_tr, target_dims = 'sdate',
                                               fun = function(x) {quantile(as.vector(x), c(1/3, 2/3), na.rm = TRUE)},
                                               output_dims = 'bin', ncores = config$ncores)$output1
          lims_ano_dcpp_tr <- multiApply::Apply(pv_dcpp_tr, target_dims = c('sdate'),
                                                fun = function(x) {quantile(as.vector(x), c(1/3, 2/3), na.rm = TRUE)},
                                                output_dims = 'bin', ncores = config$ncores)$output1
          
          pv_dcpp_probs_ev <- GetProbs_2(pv_dcpp, time_dim = 'sdate', 
                                      prob_thresholds = NULL,
                                      bin_dim_abs = 'bin', 
                                      indices_for_quantiles = NULL,
                                      memb_dim = NULL, abs_thresholds = lims_ano_dcpp_tr,
                                      ncores = config$ncores)
          pv_obs_probs_ev <- s2dv::Reorder(data = pv_dcpp_probs_ev, order = c('bin','sdate'))

          if (season == 'DJF'){
            output$pv_dcpp_DJF[[t]] <- pv_dcpp
            output$pv_obs_DJF[[t]] <- pv_obs
            output$pv_dcpp_probs_DJF[,t] <- pv_dcpp_probs_ev
            output$n_effective_dcpp_DJF[[t]] <- n_effective_dcpp
            output$n_effective_obs_DJF[[t]] <- n_effective_obs
          } else if (season == 'MAM'){
            output$pv_dcpp_MAM[[t]] <- pv_dcpp
            output$pv_obs_MAM[[t]] <- pv_obs
            output$pv_dcpp_probs_MAM[,t] <- pv_dcpp_probs_ev
            output$n_effective_dcpp_MAM[[t]] <- n_effective_dcpp
            output$n_effective_obs_MAM[[t]] <- n_effective_obs
          } else if (season == 'JJA'){
            output$pv_dcpp_JJA[[t]] <- pv_dcpp
            output$pv_obs_JJA[[t]] <- pv_obs
            output$pv_dcpp_probs_JJA[,t] <- pv_dcpp_probs_ev
            output$n_effective_dcpp_JJA[[t]] <- n_effective_dcpp
            output$n_effective_obs_JJA[[t]] <- n_effective_obs
          } else if (season == 'SON'){
            output$pv_dcpp_SON[[t]] <- pv_dcpp
            output$pv_obs_SON[[t]] <- pv_obs
            output$pv_dcpp_probs_SON[,t] <- pv_dcpp_probs_ev
            output$n_effective_dcpp_SON[[t]] <- n_effective_dcpp
            output$n_effective_obs_SON[[t]] <- n_effective_obs
          } else if (season == 'Annual'){
            output$pv_dcpp_Annual[[t]] <- pv_dcpp
            output$pv_obs_Annual[[t]] <- pv_obs
            output$pv_dcpp_probs_Annual[,t] <- pv_dcpp_probs_ev
            output$n_effective_dcpp_Annual[[t]] <- n_effective_dcpp
            output$n_effective_obs_Annual[[t]] <- n_effective_obs
          }
        }
      }
    }

  return(output)
}

    
