# setwd('/esarchive/scratch/smoreno/gitlab/capacity_factors/')
source('config.R')

args <- commandArgs(trailingOnly = TRUE)
model <- args[1]
member <- args[2]
region <- args[3]
fperiods <- '1'
model <- 'IPSL-CM6A-LR'
calibration_method <- 'quantile'

print(member)
print(model)
print(region)

if (region == 'Europe'){
  grid <- config$grid_25
} else {
  grid <- config$grid_down
}

if (identical(models,'all')){
  models <- c(config$models,config$multimodels)
} else {
  models <- unlist(strsplit(x = models, split = ','))
}

if (identical(fperiods,'all')){
  fperiods <- config$fperiods
} else {
  aux <- unlist(strsplit(x = fperiods, split = ','))
  fperiods <- list()
  for (i in 1:length(aux)){
    auxi <- unlist(strsplit(x = aux[i], split = '-'))
    fperiods[[i]] <- seq(from = auxi[1], to = auxi[length(auxi)], by = 1)
  }
}

  for (fyears in fperiods){
    time_starting_exec <- proc.time()
    
    if (length(fyears) == 1){fyears_string <- paste0('fyear',fyears[1])} else {fyears_string <- paste0('fyears',fyears[1],'-',fyears[length(fyears)])}
    sdates <- config$sdates[1] : (config$years[length(config$years)] - as.integer(fyears[length(fyears)]))
    years <- (sdates[1] + as.integer(fyears[1])) : config$years[length(config$years)]
    # indices_for_clim <- seq(from = (config$reference_period[1]-years[1]+1), length.out = length(config$reference_period)-length(fyears)+1)

    print(paste0('Starting ',model,' - ',fyears_string))
    
    ################################################################################################
    #################################### Loading the data ##########################################
    ################################################################################################
    
    print('Loading the data...')
    
    time_dim <- 'sdate'
    sys_info <- Sys.info()['nodename']
    machine_index <- substr(sys_info, 1, 2)
      if (machine_index == 'gl' | machine_index == 'as' | machine_index == 'gs' | machine_index == 'da'){
        path <- '/gpfs/scratch/bsc32/MN4/bsc32/bsc32693/data_amd/recon/era5/1hourly/$var$-r1440x721cds/$var$_$year$$month$.nc'
        path_mask <- '/gpfs/scratch/bsc32/MN4/bsc32/bsc32693/'
      } else {
        path <- '/esarchive/recon/ecmwf/era5/1hourly/$var$-r1440x721cds/$var$_$year$$month$.nc'
        path_mask <- '/esarchive/recon/ecmwf/era5/constant/lsm-r1440x721cds/'
      }

      obs_uas <- startR::Start(dat = path, var = 'uas',
                           lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                           lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                           year = paste0(config$years), month = 'all', time = 'all',
                           time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                           synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                           return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                           lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                           num_procs = 1, retrieve = TRUE)
      
      lon_uas <- attr(obs_uas,'Variables')$common[['lon']]
      lat_uas <- attr(obs_uas,'Variables')$common[['lat']]
      dates_uas <- attr(obs_uas,'Variables')$common$time
      
      # Remove 29 Feb
      indices_remove <- apply(dates_uas, 1, function(x) {which(is.na(x) == T)})
      if (config$years[1] %% 4 == 0 && (config$years[1] %% 100 != 0 || config$years[1] %% 400 == 0)){
        keep_indices <- !(seq_len(dim(obs_uas)['time']) %in% indices_remove[[2]])
      } else {
        keep_indices <- !(seq_len(dim(obs_uas)['time']) %in% indices_remove[[1]])
      }
      obs_uas <- ClimProjDiags::Subset(x = obs_uas, along = 'time', indices = keep_indices, drop = FALSE)
      dates_uas <- ClimProjDiags::Subset(x = dates_uas, along = 'time', indices = keep_indices, drop = FALSE)
      dates_first_year <- dates_uas[format(dates_uas, "%Y") == config$years[1]]
      indices_keep <- which(format(dates_first_year, "%H") %in% c("00", "06", "12", "18"))
      obs_uas <- ClimProjDiags::Subset(x = obs_uas, along = 'time', indices = indices_keep, drop = FALSE)
      dates_uas <- ClimProjDiags::Subset(x = dates_uas, along = 'time', indices = indices_keep, drop = FALSE)
      
      obs_vas <- startR::Start(dat = path, var = 'vas',
                                lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                                lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                                year = paste0(config$years), month = 'all', time = 'all',
                                time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                                synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                                return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                                lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                                num_procs = 1, retrieve = TRUE)
    
      
      lon_vas <- attr(obs_vas,'Variables')$common[['lon']]
      lat_vas <- attr(obs_vas,'Variables')$common[['lat']]
      dates_vas <- attr(obs_vas,'Variables')$common$time
      # Remove 29 Feb
      indices_remove <- apply(dates_vas, 1, function(x) {which(is.na(x) == T)})
      if (config$years[1] %% 4 == 0 && (config$years[1] %% 100 != 0 || config$years[1] %% 400 == 0)){
        keep_indices <- !(seq_len(dim(obs_vas)['time']) %in% indices_remove[[2]])
      } else {
        keep_indices <- !(seq_len(dim(obs_vas)['time']) %in% indices_remove[[1]])
      }
      obs_vas <- ClimProjDiags::Subset(x = obs_vas, along = 'time', indices = keep_indices, drop = FALSE)
      dates_vas <- ClimProjDiags::Subset(x = dates_vas, along = 'time', indices = keep_indices, drop = FALSE)
      dates_first_year <- dates_vas[format(dates_vas, "%Y") == config$years[1]]
      indices_keep <- which(format(dates_first_year, "%H") %in% c("00", "06", "12", "18"))
      obs_vas <- ClimProjDiags::Subset(x = obs_vas, along = 'time', indices = indices_keep, drop = FALSE)
      dates_vas <- ClimProjDiags::Subset(x = dates_vas, along = 'time', indices = indices_keep, drop = FALSE)
      
      obs_lons <- as.vector(lon_vas)
      obs_lats <- as.vector(lat_vas)
      indices_years <- which(config$years %in% years)
      obs_uas <- multiApply::Apply(data = obs_uas, target_dims = 'year', 
                                   fun = function(data){zoo::rollmean(x = as.vector(data), k = length(fyears), fill = NULL, align = 'left')}, 
                                   output_dims = 'year', ncores = config$ncores)$output1
      obs_uas <- s2dv::CDORemap(data_array = obs_uas, lons = obs_lons, lats = obs_lats, grid = grid, method = config$interpolation_method, crop = c(config$regions[[region]]$lon_min, config$regions[[region]]$lon_max, config$regions[[region]]$lat_min, config$regions[[region]]$lat_max), force_remap = TRUE)
      obs_vas <- multiApply::Apply(data = obs_vas, target_dims = 'year', 
                                    fun = function(data){zoo::rollmean(x = as.vector(data), k = length(fyears), fill = NULL, align = 'left')}, 
                                    output_dims = 'year', ncores = config$ncores)$output1
      obs_vas <- s2dv::CDORemap(data_array = obs_vas, lons = obs_lons, lats = obs_lats, grid = grid, method = config$interpolation_method, crop = c(config$regions[[region]]$lon_min, config$regions[[region]]$lon_max, config$regions[[region]]$lat_min, config$regions[[region]]$lat_max), force_remap = TRUE)
      obs_lons <- obs_uas$lons
      obs_lats <- obs_uas$lats
      obs_uas <- obs_uas$data_array
      obs_vas <- obs_vas$data_array
      obs_uas <- drop(obs_uas)
      obs_vas <- drop(obs_vas)
      obs_uas <- Reorder(data = obs_uas, order = c('year','lon','lat','time'))
      obs_vas <- Reorder(data = obs_vas, order = c('year','lon','lat','time'))
      sfc_wind_obs <- sqrt(obs_uas^2 + obs_vas^2)
      rm(obs_vas,obs_uas)
      
      # Corrección porque las observaciones de ERA5 tienen errores
      if (machine_index == 'gl' | machine_index == 'as' | machine_index == 'gs' | machine_index == 'da'){
        era5_path <- '/gpfs/projects/bsc32/esarchive_cache/recon/ecmwf/era5/monthly_mean/$var$_f1h-r1440x721cds/$var$_$year$$month$.nc'
        gwa100_file <- '/gpfs/scratch/bsc32/MN4/bsc32/bsc32693/data_amd/scratch/capacity_outputs/GWA_wspd100m_t170grid.nc'
      } else {
        era5_path <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h-r1440x721cds/$var$_$year$$month$.nc'  
        gwa100_file <- "/esarchive/scratch/smoreno/capacity_outputs/GWA_wspd100m_t170grid.nc"
      }
      era5_wind <- startR::Start(dat = era5_path, var = 'sfcWind',
                                 lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                                 lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                                 year = paste0(config$years), month = 'all', time = 'all',
                                 synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                                 return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                                 lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                                 num_procs = 1, retrieve = TRUE)
      lon_era5 <- attr(era5_wind,'Variables')$common[['lon']]
      lat_era5 <- attr(era5_wind,'Variables')$common[['lat']]
      lat_era5 <- as.vector(lat_era5)
      lon_era5 <- as.vector(lon_era5)
      era5_wind <- drop(era5_wind)
      era5_wind <- MeanDims(data=era5_wind, dims = c('year','month'))
      gwa.nc <- nc_open(gwa100_file)
      gwa_spd <- ncvar_get(gwa.nc,"WSPD100")
      names(dim(gwa_spd)) <- c('lon','lat')
      lat <- ncvar_get(gwa.nc,"latitude")
      lon <- ncvar_get(gwa.nc,"longitude")
      era5_wind <- s2dv::CDORemap(data_array = era5_wind, lons = lon_era5, lats = lat_era5, grid = grid, method = config$interpolation_method, 
                             crop = c(config$regions[[region]]$lon_min, config$regions[[region]]$lon_max, config$regions[[region]]$lat_min, config$regions[[region]]$lat_max), force_remap = TRUE)
      gwa_spd <- s2dv::CDORemap(data_array = gwa_spd, lons = lon, lats = lat, grid = grid, method = config$interpolation_method, 
                                    crop = c(config$regions[[region]]$lon_min, config$regions[[region]]$lon_max, config$regions[[region]]$lat_min, config$regions[[region]]$lat_max), force_remap = TRUE)
      ratio <- gwa_spd$data_array/era5_wind$data_array
      ratio_expanded <- array(ratio, dim = dim(sfc_wind_obs))
      sfc_wind_obs <- sfc_wind_obs * ratio_expanded

    if (identical(model,'MRI-ESM2-0')) { #& grepl(pattern = '_djf', x = variable) & any(fyears > 4)
      indices_fyears <- fyears[-which(fyears > 4)]
    } else if (identical(model,'MRI-ESM2-0') & any(fyears > 5)){
      indices_fyears <- fyears[-which(fyears > 5)]
    } else {indices_fyears <- fyears}
      indices_fyears <- 1

      if (model == 'EC-Earth3-i2'){
        if (machine_index == 'gl' | machine_index == 'as' | machine_index == 'gs' | machine_index == 'da'){
          path_models <- '/gpfs/scratch/bsc32/MN4/bsc32/bsc32693/data_amd/scratch/exp/CMIP6/dcppA-hindcast/EC-Earth3-i4/DCPP/EC-Earth-Consortium/EC-Earth3-i4/dcppA-hindcast/$member$/6hrPlevPt/$var$/gr/v20210910/uas_6hrPlevPt_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc'
          # path_models <- '/gpfs/projects/bsc32/esarchive_cache/exp/CMIP6/dcppA-hindcast/EC-Earth3-i4/DCPP/EC-Earth-Consortium/EC-Earth3-i4/dcppA-hindcast/$member$/6hrPlevPt/$var$/gr/v20210910/uas_6hrPlevPt_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc'
        } else {
          path_models <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/EC-Earth3/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$member$/6hrPlevPt/sfcWind/gr/v20210309/sfcWind_6hrPlevPt_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc')
        }

        for (fy in indices_fyears){
          calendar <- '6hr'
          times <- get_fdays(fyears = fy, calendar = calendar, month_init = 'Nov')
          sfc_wind <- startR::Start(dat = path_models, var = 'sfcWind',
                               lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                               lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                               sdate = paste(config$sdates), member = member, time = indices(times), #time_across = 'fyear',
                               time_across = 'aux', merge_across_dims = TRUE, aux = 'all', aux_depends = 'sdate',
                               synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                               return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                               lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                               num_procs = 1, retrieve = TRUE)
          
          lon_dcpp <- attr(sfc_wind,'Variables')$common[['lon']]
          lat_dcpp <- attr(sfc_wind,'Variables')$common[['lat']]
          dates_sfc_wind <- attr(sfc_wind,'Variables')$common$time
          lon_dcpp <- as.vector(lon_dcpp)
          lat_dcpp <- as.vector(lat_dcpp)
          sfc_wind <- s2dv::CDORemap(data_array = sfc_wind, lons = lon_dcpp , lats = lat_dcpp, grid = grid, method = config$interpolation_method, crop = c(config$regions[[region]]$lon_min, config$regions[[region]]$lon_max, config$regions[[region]]$lat_min, config$regions[[region]]$lat_max), force_remap = TRUE)
          lon_dcpp <- sfc_wind$lon
          lat_dcpp <- sfc_wind$lat
          sfc_wind <- sfc_wind$data_array
          sfc_wind <- drop(sfc_wind)
          
          days <- 1460
          if (identical(fy,indices_fyears[1])){
            sfc_wind_dcpp <- array(data = NA, dim = c('sdate' = length(config$sdates), 'lon' = length(lon_dcpp), 'lat' = length(lat_dcpp), time = days, fyear = length(indices_fyears), member = length(members)))
          }
          sfc_wind_dcpp[,,,,which(indices_fyears==fy)] <- s2dv::Reorder(data = sfc_wind, order = c('sdate','lon','lat','time','member'))
        }
        sfc_wind_dcpp <- multiApply::Apply(data = sfc_wind_dcpp, target_dims = c('fyear'), fun = mean, na.rm = FALSE, ncores = config$ncores)$output1
        
      } else {
          if (identical(model, 'EC-Earth3-i1')) {
            calendar <- '6hr'
            units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
            times <- get_fdays(fyears = fy, calendar = calendar, month_init = 'Nov')
            if (machine_index == 'gl' | machine_index == 'as' | machine_index == 'gs' | machine_index == 'da'){
              path_models <- paste0('/gpfs/scratch/bsc32/MN4/bsc32/bsc32693/data_amd/scratch/exp/CMIP6/dcppA-hindcast/EC-Earth3-i1/DCPP/EC-Earth-Consortium/EC-Earth3-i1/dcppA-hindcast/$member$/6hrPlevPt/$var$/gr/v20190713/$var$_6hrPlevPt_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc')
              # path_models <- paste0('/gpfs/projects/bsc32/esarchive_cache/exp/CMIP6/dcppA-hindcast/EC-Earth3-i1/DCPP/EC-Earth-Consortium/EC-Earth3-i1/dcppA-hindcast/$member$/6hrPlevPt/$var$/gr/v20190713/$var$_6hrPlevPt_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc')
            } else {
              path_models <- paste0('/esarchive/exp/ecearth/a1ua/cmorfiles/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$member$/6hrPlevPt/$var$/gr/v20190713/$var$_6hrPlevPt_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc')
            }
          } else if (identical(model, 'EC-Earth3-i4')) {
            member <- paste0('r',1:10,'i4p1f1')
            # calendar <- 'proleptic_gregorian'
            calendar <- '6hr'
            units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
            month_init <- 'Nov'
            if (machine_index == 'gl' | machine_index == 'as' | machine_index == 'gs' | machine_index == 'da'){
              path_models <- '/gpfs/scratch/bsc32/MN4/bsc32/bsc32693/data_amd/scratch/exp/CMIP6/dcppA-hindcast/EC-Earth3-i4/DCPP/EC-Earth-Consortium/EC-Earth3-i4/dcppA-hindcast/$member$/6hrPlevPt/$var$/gr/v20210910/$var$_6hrPlevPt_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc'
              # path_models <- '/gpfs/projects/bsc32/esarchive_cache/exp/CMIP6/dcppA-hindcast/EC-Earth3-i4/DCPP/EC-Earth-Consortium/EC-Earth3-i4/dcppA-hindcast/$member$/6hrPlevPt/$var$/gr/v20210910/$var$_6hrPlevPt_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc'
            } else {
              path_models <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/EC-Earth3/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$member$/6hrPlevPt/$var$/gr/v20210910/$var$_6hrPlevPt_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc')
            }
          } else if (identical(model, 'IPSL-CM6A-LR')) {
            # member <- paste0('r',1:10,'i1p1f1')
            calendar <- '3hr'
            units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
            month_init <- 'Jan'
            if (machine_index == 'gl' | machine_index == 'as' | machine_index == 'gs' | machine_index == 'da'){
              path_models <- paste0('/gpfs/projects/bsc32/esarchive_cache/exp/CMIP6/dcppA-hindcast/IPSL-CM6A-LR/DCPP/IPSL/IPSL-CM6A-LR/dcppA-hindcast/$member$/3hr/$var$/gr/v20200504/$var$_3hr_IPSL-CM6A-LR_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc')
            } else {
              path_models <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/IPSL-CM6A-LR/DCPP/IPSL/IPSL-CM6A-LR/dcppA-hindcast/$member$/3hr/$var$/gr/v20200504/$var$_3hr_IPSL-CM6A-LR_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc')
            }
          }
          
        for (fy in indices_fyears){
          print(fy)
          times <- get_fdays(fyears = fy, calendar = calendar, month_init = month_init)
          uas <- startR::Start(dat = path_models, var = 'uas',
                               lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                               lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                               sdate = paste(config$sdates), time = indices(times), member = member, #time_across = 'fyear',
                               # fyear = 'all', merge_across_dims = TRUE,  fyear_depends = 'sdate',
                               time_across = 'aux', merge_across_dims = TRUE, aux = 'all', aux_depends = 'sdate',
                               synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                               return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                               lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                               num_procs = 1, retrieve = TRUE)
          
          vas <- startR::Start(dat = path_models, var = 'vas',
                               lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                               lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                               sdate = paste(config$sdates), member = member, time = indices(times), #time_across = 'fyear',
                               # fyear = 'all', merge_across_dims = TRUE,  fyear_depends = 'sdate',
                               time_across = 'aux', merge_across_dims = TRUE, aux = 'all', aux_depends = 'sdate',
                               synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                               return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                               lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                               num_procs = 1, retrieve = TRUE)
          
          lon_dcpp <- attr(uas,'Variables')$common[['lon']]
          lat_dcpp <- attr(uas,'Variables')$common[['lat']]
          dates_uas <- attr(uas,'Variables')$common$time
          lon_dcpp <- as.vector(lon_dcpp)
          lat_dcpp <- as.vector(lat_dcpp)
          uas <- s2dv::CDORemap(data_array = uas, lons = lon_dcpp , lats = lat_dcpp, grid = grid, method = config$interpolation_method, crop = c(config$regions[[region]]$lon_min, config$regions[[region]]$lon_max, config$regions[[region]]$lat_min, config$regions[[region]]$lat_max), force_remap = TRUE)
          lon_dcpp <- uas$lon
          lat_dcpp <- uas$lat
          uas <- uas$data_array
          uas <- drop(uas)
          
          lon_dcpp <- attr(vas,'Variables')$common[['lon']]
          lat_dcpp <- attr(vas,'Variables')$common[['lat']]
          dates_vas <- attr(vas,'Variables')$common$time
          lon_dcpp <- as.vector(lon_dcpp)
          lat_dcpp <- as.vector(lat_dcpp)
          vas <- s2dv::CDORemap(data_array = vas, lons = lon_dcpp , lats = lat_dcpp, grid = grid, method = config$interpolation_method, crop = c(config$regions[[region]]$lon_min, config$regions[[region]]$lon_max, config$regions[[region]]$lat_min, config$regions[[region]]$lat_max), force_remap = TRUE)
          lon_dcpp <- vas$lon
          lat_dcpp <- vas$lat
          vas <- vas$data_array
          vas <- drop(vas)
          if (model == 'IPSL-CM6A-LR'){
            dates_first_year_au <- dates_uas[format(dates_uas, "%Y") == config$sdates[2]]
            indices_keep_au <- which(format(dates_first_year_au, "%H") %in% c("00", "06", "12", "18"))
            uas <- ClimProjDiags::Subset(x = uas, along = 'time', indices = indices_keep_au, drop = FALSE)
            uas_expanded <- abind::abind(uas, uas[,,,1459, drop=FALSE], along = 4)
            names(dim(uas_expanded)) <- names(dim(uas))
            uas <- uas_expanded
            vas <- ClimProjDiags::Subset(x = vas, along = 'time', indices = indices_keep_au, drop = FALSE)
            dates_uas <- ClimProjDiags::Subset(x = dates_uas, along = 'time', indices = indices_keep_au, drop = FALSE)
            vas_expanded <- abind::abind(vas, vas[,,,1459, drop=FALSE], along = 4)
            names(dim(vas_expanded)) <- names(dim(vas))
            vas <- vas_expanded
          }
          days <- as.numeric(dim(uas)["time"])
          if (identical(fy,indices_fyears[1])){
            dcpp_vas <- array(data = NA, dim = c('sdate' = length(config$sdates), 'lon' = length(lon_dcpp), 'lat' = length(lat_dcpp), time = days, fyear = length(indices_fyears)))
            dcpp_uas <- array(data = NA, dim = c('sdate' = length(config$sdates), 'lon' = length(lon_dcpp), 'lat' = length(lat_dcpp), time = days, fyear = length(indices_fyears)))
          }
          dcpp_vas[,,,,which(indices_fyears==fy)] <- s2dv::Reorder(data = vas, order = c('sdate','lon','lat','time'))
          dcpp_uas[,,,,which(indices_fyears==fy)] <- s2dv::Reorder(data = uas, order = c('sdate','lon','lat','time'))
        }
        dcpp_uas <- multiApply::Apply(data = dcpp_uas, target_dims = c('fyear'), fun = mean, na.rm = FALSE, ncores = config$ncores)$output1
        dcpp_vas <- multiApply::Apply(data = dcpp_vas, target_dims = c('fyear'), fun = mean, na.rm = FALSE, ncores = config$ncores)$output1
        sfc_wind_dcpp <- sqrt(dcpp_uas^2 + dcpp_vas^2)
      }
    rm(dcpp_uas,dcpp_vas,uas,vas)

    names(dim(sfc_wind_obs))[1] <- 'sdate'
    sdate_dim <- dim(sfc_wind_obs)['sdate']
    cross <- .make.eval.train.dexes(eval.method = 'leave-one-out', amt.points = sdate_dim,
                                    amt.points_cor = NULL, k = 1, tail.out = FALSE)
    dates_first_year_au <- dates_uas[format(dates_uas, "%Y") == config$sdates[2]]
    wind_dcpp_ev_corrected <- array(NA, dim = dim(sfc_wind_dcpp[1,,,]))
    wind_dcpp_ev_corrected <- InsertDim(data = wind_dcpp_ev_corrected, posdim = 1, lendim = 1, name = 'sdate')
    wind_dcpp_ev_corrected <- Reorder(data = wind_dcpp_ev_corrected, order = c('sdate','lon','lat','time')) #,'member'))
    years_len <- sdate_dim[['sdate']]-1
    wind_dcpp_tr_corrected <- array(NA, dim = dim(sfc_wind_dcpp[1:years_len,,,]))

    # Corrección porque la turbina está a 100m
    mask_ori <- startR::Start(dat = paste0(path_mask,'$var$.nc'), var = 'sftof',
                              lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                              lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                              synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                              return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                              lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                              num_procs = 1, retrieve = TRUE)
    lon_mask <- attr(mask_ori,'Variables')$common[['lon']]
    lat_mask <- attr(mask_ori,'Variables')$common[['lat']]
    lon_mask <- as.vector(lon_mask)
    lat_mask <- as.vector(lat_mask)
    mask <- drop(mask_ori)
    mask <- s2dv::CDORemap(data_array = mask[,], lons = lon_mask, lats = lat_mask, grid = grid, method = config$interpolation_method, 
                           crop = c(config$regions[[region]]$lon_min, config$regions[[region]]$lon_max, config$regions[[region]]$lat_min, config$regions[[region]]$lat_max), force_remap = TRUE)
    mask <- mask$data_array
    mask_extended <- array(mask, dim = c(dim(mask), dim(sfc_wind_obs)[1], dim(sfc_wind_obs)[4]))
    multiplication_factor <- ifelse(mask_extended == 0, 1.39, 1.29)
    multiplication_factor <- Reorder(data = multiplication_factor, order = c('sdate','lon','lat','time'))

    print('Calibration')
     for (t in 1:sdate_dim){
       print(t)
       wind_obs_tr <- ClimProjDiags::Subset(x = sfc_wind_obs, along = 'sdate', indices = cross[[t]]$train.dexes, drop = 'selected')
       wind_dcpp_tr <- ClimProjDiags::Subset(x = sfc_wind_dcpp, along = 'sdate', indices = cross[[t]]$train.dexes, drop = 'selected')
       wind_dcpp_ev <- ClimProjDiags::Subset(x = sfc_wind_dcpp, along = 'sdate', indices = cross[[t]]$eval.dexes, drop = FALSE)
       wind_obs_ev <- ClimProjDiags::Subset(x = sfc_wind_obs, along = 'sdate', indices = cross[[t]]$eval.dexes, drop = FALSE)
       for (hour in c("00", "06", "12", "18")){
         indices_hour <- which(format(dates_first_year_au, "%H") == hour)
         wind_dcpp_ev_hour <- ClimProjDiags::Subset(x = wind_dcpp_ev, along = 'time', indices = indices_hour, drop = FALSE)
         wind_dcpp_tr_hour <- ClimProjDiags::Subset(x = wind_dcpp_tr, along = 'time', indices = indices_hour, drop = FALSE)
         wind_obs_tr_hour <- ClimProjDiags::Subset(x = wind_obs_tr, along = 'time', indices = indices_hour, drop = FALSE)
         wind_dcpp_tr_hour_wi <- add_window_dim(wind_dcpp_tr_hour)
         wind_obs_tr_hour_wi <- add_window_dim(wind_obs_tr_hour)
         wind_dcpp_tr_hour <- QuantileMapping(exp = wind_dcpp_tr_hour_wi, obs = wind_obs_tr_hour_wi, 
                                              exp_cor = wind_dcpp_tr_hour,
                                              na.rm = TRUE,  memb_dim = NULL, sdate_dim = 'sdate', 
                                              window_dim = 'window',
                                              ncores = config$ncores, method = 'QUANT', qstep = 0.01)
         wind_dcpp_ev_hour <- QuantileMapping(exp = wind_dcpp_tr_hour_wi, obs = wind_obs_tr_hour_wi, 
                                    exp_cor = wind_dcpp_ev_hour,
                                    na.rm = TRUE,  memb_dim = NULL, sdate_dim = 'sdate', 
                                    window_dim = 'window',
                                    ncores = config$ncores, method = 'QUANT', qstep = 0.01)
         wind_dcpp_ev_hour <- Reorder(data = wind_dcpp_ev_hour, order = c('sdate','lon','lat','time'))
         wind_dcpp_tr_hour <- Reorder(data = wind_dcpp_tr_hour, order = c('sdate','lon','lat','time'))
         wind_dcpp_ev_corrected[,,,indices_hour] <- wind_dcpp_ev_hour
         wind_dcpp_tr_corrected[,,,indices_hour] <- wind_dcpp_tr_hour
       }
      
       wind_obs_tr <- wind_obs_tr * multiplication_factor[1:years_len,,,]
       multi_1 <- InsertDim(data = multiplication_factor[1,,,], posdim = 1, lendim = 1, name = 'sdate')
       wind_obs_ev <- wind_obs_ev * multi_1
       wind_dcpp_tr <- wind_dcpp_tr_corrected * multiplication_factor[1:years_len,,,]
       wind_dcpp_ev_corrected <- wind_dcpp_ev_corrected * multi_1
     }
     
    wind_dcpp_ev <- wind_dcpp_ev_corrected
    print('FINISHED :D')
    saveRDS(object = list(wind_obs_tr = wind_obs_tr,
                          wind_obs_ev = wind_obs_ev,
                          wind_dcpp_tr = wind_dcpp_tr,
                          wind_dcpp_ev = wind_dcpp_ev,
                          lon = lon_dcpp,
                          lat = lat_dcpp,
                          config = config,
                          dates = dates_uas),
            file = paste0(config$downscaling_path,'quantile_',model,'_',fyears_string,'_',member,'_',region,'.rds'))
  }
